;---------------------------------------------------------------
; Boot sector -- for 1440k floppy

%include "defs.h"

bits 16
org 0x7c00

	xor ax,ax	;Initialize segments
	mov ds,ax
        mov ss,ax
        mov sp,7c00h

;---------------------------------------------------------------
; Load cylinder 0 (boot monitor & root directory)
bootload:
	mov cx,3	;# retries
.retry
	push cx
	mov ax,0x7e0	;Load to es:bx -- address 7e00
	mov es,ax
	xor bx,bx
	xor cx,cx	;ch=0 (track)
	inc cl		;cl=2 (sector, 1-based)
	inc cl
	xor dx,dx	;dl=0 (drive)  dh=0 (head)
	mov ax,223h	;al=35 (sectors)
	int 0x13	;BIOS service $1302 - disk read
	jnc .success
	pop cx
	loop .retry
.error
        mov ax,0xb800
        mov es,ax
        mov [es:0],word 0x0445  ;Red 'E' in upper left
	jmp short $
.success
	xor al,al	;shut off floppy motor
	mov dx,0x3f2
	out dx,al

;---------------------------------------------------------------
       mov ax,0xb800
       mov es,ax
       mov [es:0],word 0x0e41  ;Yellow 'A' in upper left

;---------------------------------------------------------------
; Enter protected mode
        call graphics
        call A20
  mov [es:2],word 0x0e42

        cli
	lgdt [gdt]
	mov eax,cr0
	inc eax
	mov cr0,eax
	jmp dword 8:protected

;---------------------------------------------------------------
bits 32
protected:
	mov ax,0x10	;data seg
	mov ds,ax
	mov es,ax
	mov ss,ax
	mov esp,rp0	;return stack
	cld
mov [0xB8004],word 0x0e43

	;Note done yet:
	; Empty 8042 key buffer
	; IDT - only needed if using hardware interrupts, maybe
	; Reprogram PIC's - only necessary if using ints..

       mov edi,0xA0000                 ;gfx test
       mov eax,0x0f0f4040
       mov ecx,320/4
       rep stosd

        mov esi,7E00h                   ;Move kernel
        mov edi,START
       mov edx,edi
        mov ecx,(END-kernel)/4
        cld
        rep movsd

        mov esi,sp0     ;data stack
        call edx

        ;Reboot if it returns?

mov [0xB8000],word 0x0e2b
        jmp short $

;---------------------------------------------------------------
 align 8
gdt:	dw 0x18
	dd gdt
	dw 0
	dw 0xffff, 0, 0x9a00, 0xcf	;code
	dw 0xffff, 0, 0x9200, 0xcf	;data
;---------------------------------------------------------------

%include "init.1"

;
; *** There's still plenty of room here.. ***
;

;---------------------------------------------------------------
;End of boot sector..
times	510-($-$$)	db 0	; skip to end of sector
	dw	0xAA55		; boot sector signature

;---------------------------------------------------------------
; Sectors 2-?? contain the rest of the code
kernel:
 incbin "kern"
 align 4
END

;---------------------------------------------------------------
; End
;
times 0x2000-($-$$) db 0

;---------------------------------------------------------------
; Directory (test) (FUBAR)
dir:
;%include "dir.1"
;times 0x400-($-dir) db 0
