;---------------------------------------------------------------
; Retro running under DOS, as a .com program
;
;       Enter p-mode & move kernel to a fixed location above 1M.
;       Expects a CLEAN boot, no himem.sys, emm386 or anything.
;
;       This is sufficient for my purposes at the moment.. to be a decent
;       32-bit Forth for DOS, it would need a proper DOS extender.

;---------------------------------------------------------------
%include "defs.h"

BITS 16
ORG 100h
;---------------------------------------------------------------
;       mov ax,0xb800
;       mov es,ax
;       mov [es:0],word 0x0e41  ;Yellow 'A' in upper left

;---------------------------------------------------------------
; Enter protected mode
init:
        pushf                   ;Save DOS state
        push ds

        call graphics
        call A20

        mov bx,cs               ;Add CS to flat 32-bit addresses
        mov [_exit.cs],bx
        movzx ebx,bx
        shl ebx,4               ;ebx= linear address of CS
        or [gdt+18h+2],ebx
        or [gdt+20h+2],ebx
        mov dx,gdt
        movzx edx,dx
        add edx,ebx             ;edx= linear GDT address
        mov [gdt+2],edx
        cli                     ;Initialize pmode
        lgdt [gdt]
        mov eax,cr0
        inc ax
	mov cr0,eax
        add dword [.x],ebx
        db 0x66,0xEA            ;32-bit JMP FAR to get into pmode
.x      dd protected
        dw 8h
;---------------------------------------------------------------
BITS 32
protected:
        mov ax,10h              ;flat data segment
        mov ds,ax
        mov es,ax
        mov ss,ax
        mov esp,rp0             ;return stack

;        mov [dword 0xb8002],word 0x0e42       ;Yellow 'B'..

        lea esi,[ebx+kernel]
        mov edi,START
        push edi
        mov ecx,(END-kernel)/4
        cld
        rep movsd

        mov esi,sp0             ;data stack
        pop edi
        call edi

_exit:  mov ax,20h
        mov ds,ax
        mov ss,ax
        db 0x66,0xEA            ;16-bit JMP FAR
        dw .pm16,18h
BITS 16
.pm16:                          ;16-bit pmode
    ;Reminder: will need to restore real-mode seg regs (and IDT)
        mov eax,cr0
        dec ax
        mov cr0,eax
        db 0xEA                 ;JMP FAR back to real mode
        dw .real
.cs     dw 0
.real:  mov ax,cs               ;Restore vital registers
        mov es,ax
        mov ss,ax
        mov sp,-6       ;!!!    (well, it works :)
%ifnidni GFX,TEXT
        mov ax,3                ;Restore video mode
        int 10h
%endif
        sti
        pop ds                  ;Restore DOS state
        popf
        ret                     ;back to DOS
;---------------------------------------------------------------
 align 8
gdt:    dw 0x28
	dd gdt
	dw 0
	dw 0xffff, 0, 0x9a00, 0xcf	;code
	dw 0xffff, 0, 0x9200, 0xcf	;data
        dw 0xffff, 0, 0x9a00, 0         ;64k code, base to be set
        dw 0xffff, 0, 0x9200, 0         ;64k data, base to be set

;---------------------------------------------------------------
%include "init.1"

;---------------------------------------------------------------
 align 4
kernel:
incbin "kern"
 align 4
END
